@@ -1,10 +1,10 @@
 from vaspvis import standard
 
 
-band_folder = '../vaspvis_data/band_InAs'
-dos_folder = '../vaspvis_data/dos_InAs'
+band_folder = '../../vaspvis_data/band_InAs'
+dos_folder = '../../vaspvis_data/dos_InAs'
 
-band_folder_hse = '../vaspvis_data/band_InAs_hse'
+band_folder_hse = '../../vaspvis_data/band_InAs_hse'
 
 # ==================================================
 # -------------- Plain Band Structure --------------
@@ -21,6 +21,7 @@ standard.band_plain(
 
 standard.band_plain(
     folder=band_folder_hse,
+    output='band_plain_hse.png',
     hse=True,
     kpath='GXWLGK',
     n=20,
