@@ -8,6 +8,7 @@ from scipy.ndimage import gaussian_filter
 from functools import reduce
 import numpy as np
 import pandas as pd
+import copy
 import time
 
 
@@ -90,10 +91,17 @@ class Dos:
             spin_factor = -1
 
         tdos = self.vasprun.tdos
-        tdos_dict = {
-            'energy': np.array(tdos.energies - tdos.efermi),
-            'density': spin_factor * np.array(tdos.densities[self.spin_dict[self.spin]])
-        }
+
+        if self.spin == 'up' or self.spin == 'down':
+            tdos_dict = {
+                'energy': np.array(tdos.energies - tdos.efermi),
+                'density': spin_factor * np.array(tdos.densities[self.spin_dict[self.spin]])
+            }
+        elif self.spin == 'both':
+            tdos_dict = {
+                'energy': np.array(tdos.energies - tdos.efermi),
+                'density': np.array(tdos.densities[Spin.up]) + np.array(tdos.densities[Spin.down])
+            }
 
         return tdos_dict
 
@@ -115,18 +123,31 @@ class Dos:
 
         pdos = self.vasprun.pdos
         pdos_dict = {i: [] for i in range(len(pdos))}
-        spin = self.spin_dict[self.spin]
 
-        for (i, atom) in enumerate(pdos):
-            new_dict = {
-                i: spin_factor * atom[orbital][spin] for (i, orbital) in enumerate(atom)
-            }
+        if self.spin == 'up' or self.spin == 'down':
+            spin = self.spin_dict[self.spin]
+            for (i, atom) in enumerate(pdos):
+                new_dict = {
+                    i: spin_factor * atom[orbital][spin] for (i, orbital) in enumerate(atom)
+                }
 
-            if len(list(new_dict.keys())) == 16:
-                self.forbitals = True
+                if len(list(new_dict.keys())) == 16:
+                    self.forbitals = True
 
-            df = pd.DataFrame.from_dict(new_dict)
-            pdos_dict[i] = df
+                df = pd.DataFrame.from_dict(new_dict)
+                pdos_dict[i] = df
+
+        elif self.spin == 'both':
+            for (i, atom) in enumerate(pdos):
+                new_dict = {
+                    i: np.array(atom[orbital][Spin.up]) + np.array(atom[orbital][Spin.down]) for (i, orbital) in enumerate(atom)
+                }
+
+                if len(list(new_dict.keys())) == 16:
+                    self.forbitals = True
+
+                df = pd.DataFrame.from_dict(new_dict)
+                pdos_dict[i] = df
 
         return pdos_dict
 
@@ -284,13 +305,13 @@ class Dos:
         if len(ax.lines) == 0:
             if energyaxis == 'y':
                 ax.set_ylim(erange)
-                if spin == 'up':
+                if spin == 'up' or spin == 'both':
                     ax.set_xlim(0, np.max(density_in_plot) * 1.1)
                 elif spin == 'down':
                     ax.set_xlim(np.min(density_in_plot) * 1.1, 0)
             elif energyaxis == 'x':
                 ax.set_xlim(erange)
-                if spin == 'up':
+                if spin == 'up' or spin == 'both':
                     ax.set_ylim(0, np.max(density_in_plot) * 1.1)
                 elif spin == 'down':
                     ax.set_ylim(np.min(density_in_plot) * 1.1, 0)
@@ -299,12 +320,12 @@ class Dos:
                 ax.set_ylim(erange)
                 xlims = ax.get_xlim()
                 if xlims[0] == 0:
-                    if spin == 'up':
+                    if spin == 'up' or spin == 'both':
                         ax.set_xlim(0, np.max(density_in_plot) * 1.1)
                     elif spin == 'down':
                         ax.set_xlim(np.min(density_in_plot) * 1.1, xlims[1])
                 if xlims[1] == 0:
-                    if spin == 'up':
+                    if spin == 'up' or spin == 'both':
                         ax.set_xlim(xlims[0], np.max(density_in_plot) * 1.1)
                     elif spin == 'down':
                         ax.set_xlim(np.min(density_in_plot) * 1.1, 0)
@@ -312,16 +333,40 @@ class Dos:
                 ax.set_xlim(erange)
                 ylims = ax.get_ylim()
                 if ylims[0] == 0:
-                    if spin == 'up':
+                    if spin == 'up' or spin == 'both':
                         ax.set_ylim(0, np.max(density_in_plot) * 1.1)
                     elif spin == 'down':
                         ax.set_ylim(np.min(density_in_plot) * 1.1, ylims[1])
                 if ylims[1] == 0:
-                    if spin == 'up':
+                    if spin == 'up' or spin == 'both':
                         ax.set_ylim(ylims[0], np.max(density_in_plot) * 1.1)
                     elif spin == 'down':
                         ax.set_ylim(np.min(density_in_plot) * 1.1, 0)
 
+    def _group_layers(self):
+        poscar = self.poscar
+        sites = poscar.structure.sites
+        zvals = np.array([site.c for site in sites])
+        # zvals = np.sort(zvals)
+        unique_values = np.sort(np.unique(np.round(zvals, 3)))
+        diff = np.mean(np.diff(unique_values)) * 0.2
+
+        grouped = False
+        groups = []
+        zvals_copy = copy.deepcopy(zvals)
+        while not grouped:
+            if len(zvals_copy) > 0:
+                group_index = np.where(
+                    np.isclose(zvals, np.min(zvals_copy), atol=diff)
+                )[0]
+                zvals_copy = np.delete(zvals_copy, np.where(
+                    np.isin(zvals_copy, zvals[group_index]))[0])
+                groups.append(group_index)
+            else:
+                grouped = True
+
+        return groups
+
     def plot_plain(self, ax, linewidth=1.5, fill=True, alpha=0.3, alpha_line=1.0, sigma=0.05, energyaxis='y', color='black', erange=[-6, 6]):
         """
         This function plots the total density of states
@@ -1304,7 +1349,7 @@ class Dos:
                 handletextpad=0.1,
             )
 
-    def plot_layers(self, ax, cmap='magma', sigma=5, energyaxis='y', erange=[-6, 6]):
+    def plot_layers(self, ax, cmap='magma', sigma=5, energyaxis='y', erange=[-6, 6], vmax=0.6, fontsize=6):
         """
         This function plots a layer by layer heat map of the density
         of states.
@@ -1316,19 +1361,17 @@ class Dos:
             sigma (float): Sigma parameter for the _smearing of the heat map.
             energyaxis (str): Axis to plot the energy on. ('x' or 'y')
         """
-
-        poscar = self.poscar
-        sites = poscar.structure.sites
-        zvals = np.array([site.c for site in sites])
-        zorder = np.argsort(zvals)
         energy = self.tdos_dict['energy']
 
-        ind = np.where((erange[0] - 0.1 <= energy) & (energy <= erange[-1] + 0.1))
-        atom_index = range(len(zorder))
+        ind = np.where(
+                (erange[0] - 0.5 <= energy) & (energy <= erange[-1] + 0.5)
+        )
+        groups = self._group_layers()
+        atom_index = range(len(groups))
         energies = energy[ind]
-        densities = self._sum_atoms().to_numpy()[ind]
+        atom_densities = self._sum_atoms().to_numpy()[ind]
+        densities = np.vstack([np.sum(np.vstack(atom_densities[:,[group]]), axis=1) for group in groups])
         densities = np.transpose(densities)
-        densities = np.transpose(densities[zorder])
         densities = gaussian_filter(densities, sigma=sigma)
 
         if energyaxis == 'y':
@@ -1338,7 +1381,7 @@ class Dos:
                 densities,
                 cmap=cmap,
                 shading='gouraud',
-                vmax=0.6,
+                vmax=vmax,
             )
 
         if energyaxis == 'x':
@@ -1348,12 +1391,53 @@ class Dos:
                 np.transpose(densities),
                 cmap=cmap,
                 shading='gouraud',
-                vmax=0.6,
+                vmax=vmax,
             )
 
         fig = plt.gcf()
         cbar = fig.colorbar(im, ax=ax)
-        cbar.ax.tick_params(labelsize=6)
-        cbar.set_label('Density of States', fontsize=6)
+        cbar.ax.tick_params(labelsize=fontsize)
+        cbar.set_label('Density of States', fontsize=fontsize)
 
 
+def _main():
+    from itertools import groupby
+    import numpy as np
+    import copy
+    # poscar = self.poscar
+    poscar = Poscar.from_file(
+        '../../../../projects/Ogre/script/pas/POSCAR-20', check_for_POTCAR=False)
+    sites = poscar.structure.sites
+    zvals = np.array([site.c for site in sites])
+    # zvals = np.sort(zvals)
+    unique_values = np.sort(np.unique(np.round(zvals, 3)))
+    diff = np.mean(np.diff(unique_values)) * 0.2
+
+    grouped = False
+    groups = []
+    zvals_copy = copy.deepcopy(zvals)
+    while not grouped:
+        if len(zvals_copy) > 0:
+            group_index = np.where(
+                np.isclose(zvals, np.min(zvals_copy), atol=diff)
+            )[0]
+            zvals_copy = np.delete(zvals_copy, np.where(
+                np.isin(zvals_copy, zvals[group_index]))[0])
+            groups.append(group_index)
+        else:
+            grouped = True
+
+    [print(group) for group in groups]
+
+    # if to_delete is None:
+    # top_index = np.where(
+    # np.isclose(z_pos, np.max(z_pos), atol=tol)
+    # )[0]
+    # else:
+    # top_index = np.where(
+    # np.isclose(z_pos, np.max(np.delete(z_pos, to_delete)), atol=tol)
+    # )[0]
+
+
+if __name__ == '__main__':
+    _main()
